"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fuse_js_1 = __importDefault(require("fuse.js"));
const React = __importStar(require("react"));
const formatResults_1 = require("./formatResults");
class FuzzyHighlighter extends React.Component {
    constructor() {
        super(...arguments);
        this.state = {
            results: [],
            cache: {},
            info: {
                timing: 0
            }
        };
    }
    componentDidMount() {
        const { data, options } = this.props;
        this.fuse = new fuse_js_1.default(data, Object.assign({}, options, { shouldSort: true, includeMatches: true }));
        this.search();
    }
    componentDidUpdate(prevProps) {
        if (prevProps.query !== this.props.query) {
            this.search();
        }
        if (prevProps.data !== this.props.data) {
            this.setState({
                cache: {}
            }, () => {
                this.search();
            });
        }
    }
    render() {
        const { children } = this.props;
        const { results, info } = this.state;
        if (children) {
            return children({
                results,
                formattedResults: formatResults_1.formatResults(results),
                timing: info.timing
            });
        }
        return null;
    }
    search() {
        const { query } = this.props;
        const { cache } = this.state;
        if (Object(cache).hasOwnProperty(query)) {
            return this.setState({
                results: cache[query],
                info: {
                    timing: 0
                }
            });
        }
        const start = window.performance.now();
        const search = this.fuse.search(query);
        const results = search;
        const end = window.performance.now();
        this.setState({
            results,
            cache: Object.assign({}, cache, { [query]: results }),
            info: {
                timing: parseFloat((end - start).toFixed(3))
            }
        });
    }
}
exports.default = FuzzyHighlighter;
